part of model_base;

class Increment extends ModelBase
{
  static const Duration duration = const Duration(minutes: 10);

  Increment(DateTime start_time, List<String> user_ids) : super()
  {
    startTime = start_time;
    endTime = startTime.add(duration);
    userStates = user_ids.map((id) => new UserState(id)).toList(growable: false);
  }

  Increment.from(Increment other)
  {
    startTime = other.startTime;
    endTime = other.endTime;
    userStates = new List.from(other.userStates);
    availableRoomIds = new List.from(other.availableRoomIds);
  }

  @override
  Increment.decode(Map<String, dynamic> d)
  {
    startTime = DateTime.parse(d["start_time"]);
    endTime = DateTime.parse(d["end_time"]);
    List<Map<String, String>> userStateTable = d["user_states"];
    userStates = userStateTable.map((row) => new UserState.decode(row)).toList(growable: false);
  }

  @override
  Map<String, dynamic> get encoded
  {
    Map<String, dynamic> output = super.encoded;
    output["user_states"] = userStates.map((user_state) => user_state.encoded).toList(growable: false);
    return output;
  }

  @override
  Map<String, String> get toTable
  {
    Map<String, String> table = new Map();
    table[ModelBase.phrase.get(['date'])] = ModelBase.timestampFormat(startTime);
    return table;
  }

  /// This list is not stored in the database, as it depends on context (selected service and salon)
  List<String> availableRoomIds = new List();

  Map<String, dynamic> _data = new Map();
  DateTime get startTime => _data["start_time"];
  DateTime get endTime => _data["end_time"];
  List<UserState> get userStates => _data["user_states"];

  UserState getUserState(String user_id) => userStates.firstWhere((us) => us.userId == user_id, orElse: null);

  void set startTime(DateTime value) { _data["start_time"] = value; }
  void set endTime(DateTime value) { _data["end_time"] = value; }
  void set userStates(List<UserState> value) { _data["user_states"] = value; }
}

class UserState
{
  UserState(this.userId)
  {
    bookingId = null;
    state = null;
  }

  UserState.decode(Map<String, String> data) : userId = data["user_id"]
  {
    bookingId = data["booking_id"];
    state = data["state"];
  }

  Map<String, String> get encoded => {"user_id" : userId, "booking_id" : bookingId, "state" : state};

  final String userId;
  String bookingId;
  String state;
}